; this program builds an input file for the
; dot program that generates a plot of a Scheme tree

; this program requires that the selected
; language be MzScheme due to the output

; write the information needed for the graph header

(define (plot-tree t . filename)
  
  (define (build-plot)
    (build-header t)
    (build-body t)
    (build-footer t))
  
  (define (build-header t)
    (display "digraph g {")
    (newline)
    (display "label=\"")
    (write t)
    (display "\"")
    (newline)
    (display "node [height=.2]")
    (newline)
    (display "center=true")
    (newline))
  
  (define (plot-atom name value)
    (display (string-append name " [shape=box,width=.2,label=\""))
    (display value)
    (display "\"]")
    (newline))
  
  (define (plot-pair name value)
    (define (ptr->string ptr)
      (if (null? ptr) "nil" "---"))
    (display (string-append 
              name 
              " [shape=record,label=\"<f0>"
              (ptr->string (car value))
              "|<f1>"
              (ptr->string (cdr value))
              "\"]"))
    (newline))
  
  (define (plot-pair-links name value)
    (if (not (null? (car value)))
        (begin
          (display (string-append "\"" name "\":f0 -> " 
                                  (string-append name "l")))
          (newline)))
    (if (not (null? (cdr value)))
        (begin
          (display (string-append "\"" name "\":f1 -> "
                                  (string-append name "r")))
          (newline))))
  
  ; plot the body as a nested list of cons boxes
  
  (define (build-body t)
    (define (traverse-tree nodename tree)
      (cond
        ((null? tree) )
        ((not (pair? tree)) (plot-atom nodename tree))
        (else (begin
                (plot-pair nodename tree)
                (traverse-tree (string-append nodename "l") (car tree))
                (traverse-tree (string-append nodename "r") (cdr tree))
                (plot-pair-links nodename tree)))))
    (traverse-tree "root" t))
  
  ; write the graph footer for tree t
  
  (define (build-footer t)
    (display "}")
    (newline))
  
  ; initialize output file and run (finally!)
  
  (if (null? filename)
      (build-plot)
      (let ((file (car filename)))
        (if (file-exists? file)
            (delete-file file))
        (with-output-to-file file build-plot))))
